package com.example.course.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

/**
 * 微信服务工具类
 */
@Slf4j
@Component
public class WxService {

    @Value("${wx.appId}")
    private String appId;

    @Value("${wx.appSecret}")
    private String appSecret;

    private final String WX_AUTH_URL = "https://api.weixin.qq.com/sns/jscode2session";
    private final RestTemplate restTemplate = new RestTemplate();

    /**
     * 通过code获取微信用户信息
     * @param code 微信授权码
     * @return 微信用户信息
     */
    public JSONObject getWxUserInfo(String code) {
        try {
            // 构建请求URL
            String url = String.format("%s?appid=%s&secret=%s&js_code=%s&grant_type=authorization_code",
                    WX_AUTH_URL, appId, appSecret, code);

            // 发送GET请求到微信服务器
            ResponseEntity<String> response = restTemplate.getForEntity(url, String.class);
            String body = response.getBody();

            if (body != null && !body.isEmpty()) {
                JSONObject jsonObject = JSON.parseObject(body);

                // 检查返回结果是否包含错误信息
                if (jsonObject.containsKey("errcode") && jsonObject.getInteger("errcode") != 0) {
                    log.error("获取微信用户信息失败: {}", jsonObject.getString("errmsg"));
                    return null;
                }

                return jsonObject;
            }
        } catch (Exception e) {
            log.error("调用微信接口失败", e);
        }
        return null;
    }

    /**
     * 验证微信签名
     * @param signature 微信签名
     * @param timestamp 时间戳
     * @param nonce 随机字符串
     * @param data 数据
     * @return 验证结果
     */
    public boolean checkSignature(String signature, String timestamp, String nonce, String data) {
        try {
            // TODO: 实现微信签名验证逻辑
            // 1. 将timestamp、nonce、data进行字典序排序
            // 2. 将三个参数字符串拼接成一个字符串
            // 3. 进行sha1加密
            // 4. 将加密后的字符串与signature对比
            return true;
        } catch (Exception e) {
            log.error("验证微信签名失败", e);
            return false;
        }
    }

    /**
     * 解密微信数据
     * @param encryptedData 加密数据
     * @param sessionKey 会话密钥
     * @param iv 初始向量
     * @return 解密后的数据
     */
    public JSONObject decryptWxData(String encryptedData, String sessionKey, String iv) {
        try {
            // TODO: 实现微信数据解密逻辑
            // 1. 使用Base64解码encryptedData和sessionKey
            // 2. 使用AES-128-CBC解密，使用sessionKey作为密钥，iv作为初始向量
            // 3. 解析解密后的JSON数据
            return null;
        } catch (Exception e) {
            log.error("解密微信数据失败", e);
            return null;
        }
    }

    /**
     * 获取微信接口调用凭证
     * @return 接口调用凭证
     */
    public String getAccessToken() {
        try {
            String url = String.format("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s",
                    appId, appSecret);

            ResponseEntity<String> response = restTemplate.getForEntity(url, String.class);
            String body = response.getBody();

            if (body != null && !body.isEmpty()) {
                JSONObject jsonObject = JSON.parseObject(body);
                if (!jsonObject.containsKey("errcode")) {
                    return jsonObject.getString("access_token");
                }
            }
        } catch (Exception e) {
            log.error("获取微信接口调用凭证失败", e);
        }
        return null;
    }
}
